/*
 * Copyright 2007 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.hibernate4gwt.rebind;

/**
 * Helper class for proxy generators
 * @author bruno.marchesson
 *
 */
public class ProxyGeneratorHelper
{
//	----
	// Constants
	//----
	/**
	 * Constant for Proxy Suffix
	 */
	private static final String PROXY_SUFFIX = "_h4gwt_Proxy";
	
	//-------------------------------------------------------------------------
	//
	// Static methods
	//
	//-------------------------------------------------------------------------
	/**
	 * @return the computed proxy name
	 */
	public static final String computeProxyName(String className)
	{
		return className + PROXY_SUFFIX;
	}
	
	/**
	 * Indicates if the argument class name denotes a proxy class
	 */
	public static final boolean isProxy(String className)
	{
		return className.endsWith(PROXY_SUFFIX);
	}
	
	/**
	 * Indicates if the argument class name denotes a proxy class
	 */
	public static final String getProxifiedClass(String proxyName)
	{
		return proxyName.substring(0, proxyName.length() - PROXY_SUFFIX.length());
	}
	
}
